#include <stdio.h>
#include <stdlib.h>
#include <float.h>

#define MAXV 1024
#define MAXE 131072
#define MAXSV 1048576
#define MAXS 1024

typedef struct Vex TVex;
typedef struct Edge TEdge;
typedef struct SVex TSVex;

struct Vex {
       TEdge *edges;
};

struct Edge {
       int v, w;
       TEdge *next;
};

struct SVex {
       int h;
       double k;
};

int n, m;
TVex vg[MAXV];
TSVex vs[MAXSV];
int heap[MAXSV], hsize;

void add_edge(int a, int b, int w) {
     TEdge *e;

     e = (TEdge*)malloc(sizeof(TEdge));
     e->v = b;
     e->w = w;
     e->next = vg[a].edges;
     vg[a].edges = e;
}

void moveup(int e) {
     int p, v;

     v = heap[e];
     while (e != 0) {
           p = (e-1) / 2;
           if (vs[heap[p]].k > vs[v].k) {
              heap[e] = heap[p];
              vs[heap[e]].h = e;
              e = p;
           }
           else
               break;
     }
     heap[e] = v;
     vs[v].h = e;
}

void movedown(int e) {
     int c1, c2, v;

     v = heap[e];
     while (1) {
           if ((c1 = 2*e+1) < hsize) {
              if ((c2 = c1 + 1) < hsize) {
                 if (vs[heap[c2]].k < vs[heap[c1]].k)
                    c1 = c2;
              }
              if (vs[heap[c1]].k < vs[heap[e]].k) {
                 heap[e] = heap[c1];
                 vs[heap[e]].h = e;
                 e = c1;
              }
              else
                  break;
           }
           else
               break;
     }
     heap[e] = v;
     vs[v].h = e;
}

int getbest(void) {
    int r;

    r = heap[0];
    heap[0] = heap[--hsize];
    vs[heap[0]].h = 0;
    movedown(0);

    return r;
}

void dijkstra(int s) {
     int i, j, v, t, p, r;
     TEdge *e;
     double l, g;

     for (i = 0; i < n; ++i) {
         v = i * MAXS;
         for (j = 0; j < MAXS; j++) {
             vs[v].k = DBL_MAX * 0.1;
             heap[hsize] = v;
             vs[v].h = hsize++;
             v++;
         }
     }

     v = s*MAXS;
     vs[v].k = 0.0;
     moveup(vs[v].h);

     while (hsize > 0) {
           v = getbest();
           t = v / MAXS;
           p = v % MAXS;

           for (e = vg[t].edges; e; e = e->next) {
               l = e->w * 0.5 - p * 0.5;
               if (l < 0.0)
                  l = 0.0;
               if (vs[r = e->v * MAXS + e->w / 2].k > (g = vs[v].k + l)) {
                  vs[r].k = g;
                  moveup(vs[r].h);
               }
           }
     }
}

int main(void) {
    FILE *fin, *fout;
    int i, x, y, p, a, b;
    double km;

    fin = fopen("hill.inp", "r");
    fscanf(fin, "%d %d\n", &n, &m);
    for (i = 0; i < m; ++i) {
        fscanf(fin, "%d %d %d\n", &x, &y, &p);
        --x;
        --y;
        add_edge(x, y, 2*p);
        add_edge(y, x, 2*p);
    }
    fscanf(fin, "%d %d\n", &a, &b);
    --a;
    --b;
    fclose(fin);

    dijkstra(a);

    km = DBL_MAX * 0.1;
    for (i = 0; i < MAXS; ++i)
        if (vs[x = b*MAXS+i].k < km)
           km = vs[x].k;

    fout = fopen("hill.out", "w");
    if (km < DBL_MAX * 0.01)
       fprintf(fout, "%.2f\n", km * 0.5);
    else
        fprintf(fout, "0\n");
    fclose(fout);
    
    return 0;
}
